@@ -1,9 +1,9 @@
 package Net::DNS::Domain;
 
 #
-# $Id: Domain.pm 1527 2017-01-18 21:42:48Z willem $
+# $Id: Domain.pm 1561 2017-04-19 13:08:13Z willem $
 #
-our $VERSION = (qw$LastChangedRevision: 1527 $)[1];
+our $VERSION = (qw$LastChangedRevision: 1561 $)[1];
 
 
 =head1 NAME
@@ -43,18 +43,14 @@ use Carp;
 
 use constant ASCII => ref eval {
 	require Encode;
-	Encode::find_encoding('ascii');				# encoding object
+	Encode::find_encoding('ascii');
 };
 
-use constant LIBUTF8 => scalar eval {
-	Encode::decode_utf8( chr(91) ) eq '[';			# not UTF-EBCDIC  [see UTR#16 3.6]
+use constant UTF8 => scalar eval {	## not UTF-EBCDIC  [see UTR#16 3.6]
+	Encode::encode_utf8( chr(182) ) eq pack( 'H*', 'C2B6' );
 };
 
-use constant UTF8 => ref eval {
-	LIBUTF8 && Encode::find_encoding('utf8');		# encoding object
-};
-
-use constant LIBIDN => UTF8 && defined eval 'require Net::LibIDN';
+use constant LIBIDN => defined eval { require Net::LibIDN; };
 
 
 # perlcc: address of encoding objects must be determined at runtime
@@ -89,6 +85,8 @@ for zone files described in RFC1035.
 
 =cut
 
+my ( %escape, %unescape );		## precalculated ASCII escape tables
+
 our $ORIGIN;
 my ( $cache1, $cache2, $limit ) = ( {}, {}, 100 );
 
@@ -104,20 +102,25 @@ sub new {
 
 	my $self = bless {}, $class;
 
-	$s =~ s/\\\\/\\092/g;					# disguise escaped escape
-	$s =~ s/\\\./\\046/g;					# disguise escaped dot
+	local $_ = _encode_ascii($s);
 
-	my $label = $self->{label} = $s eq '@' ? [] : [split /\056/, _encode_ascii($s)];
+	s/\134\134/\134\060\071\062/g;				# disguise escaped escape
+	s/\134\056/\134\060\064\066/g;				# disguise escaped dot
 
-	foreach my $l (@$label) {
-		$l = _unescape($l) if $l =~ /\\/;
-		croak 'empty domain label' unless my $size = length($l);
-		( substr( $l, 63 ) = '', carp 'domain label truncated' ) if $size > 63;
+	my $label = $self->{label} = $_ eq "\100" ? [] : [split /\056/];
+
+	foreach (@$label) {
+		s/\134([\060-\071]{3})/$unescape{$1}/eg;	# numeric escape
+		s/\134(.)/$1/g;					# character escape
+		croak 'empty domain label' unless length;
+		next unless length > 63;
+		substr( $_, 63 ) = '';
+		carp 'domain label truncated';
 	}
 
 	$$cache1{$k} = $self;					# cache object reference
 
-	return $self if $s =~ /\.$/;				# fully qualified name
+	return $self if /\056$/;				# fully qualified name
 	$self->{origin} = $ORIGIN || return $self;		# dynamically scoped $ORIGIN
 	return $self;
 }
@@ -138,20 +141,16 @@ numerical escape sequence.
 
 =cut
 
-my $dot = '.';
-
 sub name {
 	my ($self) = @_;
 
 	return $self->{name} if defined $self->{name};
 	return unless defined wantarray;
 
-	my $lref = $self->{label};
-	my $head = _decode_ascii( join chr(46), map _escape($_), @$lref );
-	my $tail = $self->{origin} || return $self->{name} = $head || $dot;
-	return $self->{name} = $tail->name unless length $head;
-	my $suffix = $tail->name;
-	return $self->{name} = $suffix eq $dot ? $head : join $dot, $head, $suffix;
+	my @label = map { s/([^\055\101-\132\141-\172\060-\071])/$escape{$1}/eg; $_ } $self->_wire;
+
+	return $self->{name} = '.' unless scalar @label;
+	$self->{name} = _decode_ascii( join chr(46), @label );
 }
 
 
@@ -166,7 +165,7 @@ name, including the trailing dot.
 
 sub fqdn {
 	my $name = &name;
-	return $name =~ /[$dot]$/o ? $name : $name . $dot;	# append trailing dot
+	return $name =~ /[.]$/ ? $name : $name . '.';		# append trailing dot
 }
 
 
@@ -185,7 +184,7 @@ Net::LibIDN module is installed.
 sub xname {
 	my $name = &name;
 
-	if ( LIBIDN && $name =~ /xn--/ ) {
+	if ( LIBIDN && UTF8 && $name =~ /xn--/ ) {
 		my $self = shift;
 		return $self->{xname} if defined $self->{xname};
 		return $self->{xname} = $utf8->decode( Net::LibIDN::idn_to_unicode $name, 'utf-8' );
@@ -203,11 +202,19 @@ Identifies the domain by means of a list of domain labels.
 =cut
 
 sub label {
+	map {
+		s/([^\055\101-\132\141-\172\060-\071])/$escape{$1}/eg;
+		_decode_ascii($_)
+	} shift->_wire;
+}
+
+
+sub _wire {
 	my $self = shift;
 
-	my @head = map _decode_ascii( _escape($_) ), @{$self->{label}};
-	my $tail = $self->{origin} || return (@head);
-	return ( @head, $tail->label );
+	my $label = $self->{label};
+	my $origin = $self->{origin} || return (@$label);
+	return ( @$label, $origin->_wire );
 }
 
 
@@ -225,7 +232,7 @@ represented by the appropriate escape sequence.
 
 sub string {
 	( my $name = &name ) =~ s/(["'\$();@])/\\$1/;		# escape special char
-	return $name =~ /[$dot]$/o ? $name : $name . $dot;	# append trailing dot
+	return $name =~ /[.]$/ ? $name : $name . '.';		# append trailing dot
 }
 
 
@@ -263,7 +270,7 @@ sub _decode_ascii {			## translate ASCII to perl string
 	# partial transliteration for non-ASCII character encodings
 	$s =~ tr
 	[\040-\176\000-\377]
-	[ !"#$%&'()*+,-./0-9:;<=>?@A-Z\[\\\]^_`a-z{|}~?] unless ASCII;
+	[ !"#$%&'()*+,\-./0-9:;<=>?@A-Z\[\\\]^_`a-z{|}~?] unless ASCII;
 
 	my $z = length substr $s, 0, 0;				# pre-5.18 taint workaround
 	return ASCII ? pack( "a* x$z", $ascii->decode($s) ) : $s;
@@ -275,7 +282,7 @@ sub _encode_ascii {			## translate perl string to ASCII
 
 	my $z = length substr $s, 0, 0;				# pre-5.18 taint workaround
 
-	if ( LIBIDN && $s =~ /[^\000-\177]/ ) {
+	if ( LIBIDN && UTF8 && $s =~ /[^\000-\177]/ ) {
 		my $xn = Net::LibIDN::idn_to_ascii( $s, 'utf-8' );
 		croak 'invalid name' unless $xn;
 		return pack "a* x$z", $xn;
@@ -283,14 +290,14 @@ sub _encode_ascii {			## translate perl string to ASCII
 
 	# partial transliteration for non-ASCII character encodings
 	$s =~ tr
-	[ !"#$%&'()*+,-./0-9:;<=>?@A-Z\[\\\]^_`a-z{|}~\000-\377]
+	[ !"#$%&'()*+,\-./0-9:;<=>?@A-Z\[\\\]^_`a-z{|}~\000-\377]
 	[\040-\176\077] unless ASCII;
 
 	return ASCII ? pack( "a* x$z", $ascii->encode($s) ) : $s;
 }
 
 
-my %esc = eval {			## precalculated ASCII escape table
+%escape = eval {			## precalculated ASCII escape table
 	my %table;
 
 	foreach ( 33 .. 126 ) {					# ASCII printable
@@ -315,14 +322,7 @@ my %esc = eval {			## precalculated ASCII escape table
 };
 
 
-sub _escape {				## Insert escape sequences in string
-	my $s = shift;
-	$s =~ s/([^\055\101-\132\141-\172\060-\071])/$esc{$1}/eg;
-	return $s;
-}
-
-
-my %unesc = eval {			## precalculated numeric escape table
+%unescape = eval {			## precalculated numeric escape table
 	my %table;
 
 	foreach my $n ( 0 .. 255 ) {
@@ -331,21 +331,14 @@ my %unesc = eval {			## precalculated numeric escape table
 		# partial transliteration for non-ASCII character encodings
 		$key =~ tr [0-9] [\060-\071];
 
-		$table{$key} = pack 'C*', $n, $n == 92 ? ($n) : ();
+		$table{$key} = pack 'C', $n;
+		$table{$key} = pack 'C2', 92, $n if $n == 92;	   # escaped escape
 	}
 
 	return %table;
 };
 
 
-sub _unescape {				## Remove escape sequences in string
-	my $s = shift;
-	$s =~ s/\134([\060-\071]{3})/$unesc{$1}/eg;		# numeric escape
-	$s =~ s/\134(.)/$1/g;					# character escape
-	return $s;
-}
-
-
 1;
 __END__
 
@@ -364,7 +357,7 @@ compiled into the code.
 
 =head1 COPYRIGHT
 
-Copyright (c)2009-2011 Dick Franks.
+Copyright (c)2009-2011,2017 Dick Franks.
 
 All rights reserved.
 
