@@ -1,13 +1,13 @@
 package Net::DNS::RR::DNAME;
 
 #
-# $Id: DNAME.pm 1406 2015-10-05 08:25:49Z willem $
+# $Id: DNAME.pm 1528 2017-01-18 21:44:58Z willem $
 #
-use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1406 $)[1];
+our $VERSION = (qw$LastChangedRevision: 1528 $)[1];
 
 
 use strict;
+use warnings;
 use base qw(Net::DNS::RR);
 
 =head1 NAME
@@ -125,6 +125,6 @@ DEALINGS IN THE SOFTWARE.
 
 =head1 SEE ALSO
 
-L<perl>, L<Net::DNS>, L<Net::DNS::RR>, RFC2672
+L<perl>, L<Net::DNS>, L<Net::DNS::RR>, RFC6672
 
 =cut
