@@ -1,13 +1,13 @@
 package Net::DNS::RR::HINFO;
 
 #
-# $Id: HINFO.pm 1382 2015-08-26 20:39:33Z willem $
+# $Id: HINFO.pm 1528 2017-01-18 21:44:58Z willem $
 #
-use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1382 $)[1];
+our $VERSION = (qw$LastChangedRevision: 1528 $)[1];
 
 
 use strict;
+use warnings;
 use base qw(Net::DNS::RR);
 
 =head1 NAME
@@ -59,7 +59,7 @@ sub cpu {
 	my $self = shift;
 
 	$self->{cpu} = new Net::DNS::Text(shift) if scalar @_;
-	$self->{cpu}->value if defined wantarray && $self->{cpu};
+	$self->{cpu}->value if $self->{cpu};
 }
 
 
@@ -67,7 +67,7 @@ sub os {
 	my $self = shift;
 
 	$self->{os} = new Net::DNS::Text(shift) if scalar @_;
-	$self->{os}->value if defined wantarray && $self->{os};
+	$self->{os}->value if $self->{os};
 }
 
 
