@@ -1,13 +1,13 @@
 package Net::DNS::RR::L32;
 
 #
-# $Id: L32.pm 1390 2015-09-11 11:42:11Z willem $
+# $Id: L32.pm 1528 2017-01-18 21:44:58Z willem $
 #
-use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1390 $)[1];
+our $VERSION = (qw$LastChangedRevision: 1528 $)[1];
 
 
 use strict;
+use warnings;
 use base qw(Net::DNS::RR);
 
 =head1 NAME
@@ -66,8 +66,7 @@ sub locator32 {
 
 	$self->{locator32} = pack 'C* @4', split /\./, $prfx if defined $prfx;
 
-	join '.', unpack 'C4', $self->{locator32}
-			if defined wantarray && $self->{locator32};
+	join '.', unpack 'C4', $self->{locator32} if $self->{locator32};
 }
 
 
