@@ -1,13 +1,13 @@
 package Net::DNS::RR::OPENPGPKEY;
 
 #
-# $Id: OPENPGPKEY.pm 1499 2016-08-24 11:29:20Z willem $
+# $Id: OPENPGPKEY.pm 1528 2017-01-18 21:44:58Z willem $
 #
-use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1499 $)[1];
+our $VERSION = (qw$LastChangedRevision: 1528 $)[1];
 
 
 use strict;
+use warnings;
 use base qw(Net::DNS::RR);
 
 =head1 NAME
