@@ -15,9 +15,9 @@ sub UNITCHECK {				## restore %SIG after compilation
 package Net::DNS::RR::SIG;
 
 #
-# $Id: SIG.pm 1552 2017-03-13 09:44:07Z willem $
+# $Id: SIG.pm 1582 2017-07-07 21:45:14Z willem $
 #
-our $VERSION = (qw$LastChangedRevision: 1552 $)[1];
+our $VERSION = (qw$LastChangedRevision: 1582 $)[1];
 
 
 use strict;
@@ -123,7 +123,7 @@ sub _defaults {				## specify RR attribute default values
 #
 {
 	my @algbyname = (
-		'DELETE'	     => 0,			# [RFC4034][RFC4398][RFC8087]
+		'DELETE'	     => 0,			# [RFC4034][RFC4398][RFC8078]
 		'RSAMD5'	     => 1,			# [RFC3110][RFC4034]
 		'DH'		     => 2,			# [RFC2539]
 		'DSA'		     => 3,			# [RFC3755][RFC2536]
@@ -138,8 +138,8 @@ sub _defaults {				## specify RR attribute default values
 		'ECC-GOST'	     => 12,			# [RFC5933]
 		'ECDSAP256SHA256'    => 13,			# [RFC6605]
 		'ECDSAP384SHA384'    => 14,			# [RFC6605]
-		'Ed25519'	     => 15,			# [RFC8080]
-		'Ed448'		     => 16,			# [RFC8080]
+		'ED25519'	     => 15,			# [RFC8080]
+		'ED448'		     => 16,			# [RFC8080]
 
 		'INDIRECT'   => 252,				# [RFC4034]
 		'PRIVATEDNS' => 253,				# [RFC4034]
@@ -149,21 +149,13 @@ sub _defaults {				## specify RR attribute default values
 
 	my %algbyval = reverse @algbyname;
 
-	my $map = sub {
-		my $arg = shift;
-		unless ( $arg =~ /^\d/ ) {
-			$arg =~ s/[^A-Za-z0-9]//g;		# synthetic key
-			return uc $arg;
-		}
-		my @map = ( $arg, "$arg" => $arg );		# also accept number
-	};
-
-	my %algbyname = map &$map($_), @algbyname;
+	my @algrehash = map /^\d/ ? ($_) x 3 : do { s/[\W]//g; uc($_) }, @algbyname;
+	my %algbyname = @algrehash;    # work around broken cperl
 
 	sub _algbyname {
 		my $arg = shift;
 		my $key = uc $arg;				# synthetic key
-		$key =~ s/[^A-Z0-9]//g;				# strip non-alphanumerics
+		$key =~ s/[\W_]//g;				# strip non-alphanumerics
 		my $val = $algbyname{$key};
 		return $val if defined $val;
 		return $key =~ /^\d/ ? $arg : croak "unknown algorithm $arg";
