@@ -1,13 +1,13 @@
 package Net::DNS::RR::URI;
 
 #
-# $Id: URI.pm 1382 2015-08-26 20:39:33Z willem $
+# $Id: URI.pm 1528 2017-01-18 21:44:58Z willem $
 #
-use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1382 $)[1];
+our $VERSION = (qw$LastChangedRevision: 1528 $)[1];
 
 
 use strict;
+use warnings;
 use base qw(Net::DNS::RR);
 
 =head1 NAME
@@ -76,7 +76,7 @@ sub target {
 	my $self = shift;
 
 	$self->{target} = new Net::DNS::Text(shift) if scalar @_;
-	$self->{target}->value if defined wantarray && $self->{target};
+	$self->{target}->value if $self->{target};
 }
 
 
